@@ -24,13 +24,14 @@
 #include <goffice/graph/gog-graph-impl.h> /* for gog_graph_request_update */
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/data/go-data.h>
-#include <goffice/gtk/goffice-gtk.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
 #include <gtk/gtkcombobox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtknotebook.h>
@@ -38,6 +39,9 @@
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkwidget.h>
+#endif
+
+/*****************************************************************************/
 
 GogEditor *
 gog_editor_new (void)
@@ -72,6 +76,7 @@ gog_editor_set_store_page (GogEditor *editor, unsigned *store_page)
 	editor->store_page = store_page;
 }
 
+#ifdef GOFFICE_WITH_GTK
 static void
 cb_switch_page (G_GNUC_UNUSED GtkNotebook *n, G_GNUC_UNUSED GtkNotebookPage *p,
 		guint page_num, guint *store_page)
@@ -119,6 +124,7 @@ gog_editor_get_notebook (GogEditor *editor)
 
 	return notebook;
 }
+#endif
 
 void
 gog_editor_free (GogEditor *editor)
@@ -182,6 +188,7 @@ enum {
 	CHILDREN_REORDERED,
 	NAME_CHANGED,
 	CHANGED,
+	UPDATE_EDITOR,
 	LAST_SIGNAL
 };
 static gulong gog_object_signals [LAST_SIGNAL] = { 0, };
@@ -351,21 +358,25 @@ gog_object_get_property (GObject *obj, guint param_id,
 	}
 }
 
+#ifdef GOFFICE_WITH_GTK
 typedef struct {
 	GtkWidget	*x_spin, *y_spin, *w_spin, *h_spin;
-	GtkWidget	*manual_toggle;
+	GtkWidget	*position_select_combo;
+	GtkWidget	*position_notebook;
 	GogObject	*gobj;
 	GladeXML	*gui;
+	gulong		 update_editor_handler;
 } ObjectPrefState;
 
 static void
 object_pref_state_free (ObjectPrefState *state) 
 {
+	g_signal_handler_disconnect (state->gobj, state->update_editor_handler);
 	g_object_unref (state->gobj);
 	g_object_unref (state->gui);
+	g_free (state);
 }
 
-
 static void
 cb_compass_changed (GtkComboBox *combo, ObjectPrefState *state)
 {
@@ -398,16 +409,28 @@ cb_position_changed (GtkWidget *spin, ObjectPrefState *state)
 	else if (spin == state->h_spin)
 		pos.h = value;
 	gog_object_set_manual_position (state->gobj, &pos);
-	if (state->manual_toggle != NULL)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (state->manual_toggle), TRUE);
 }
 
 static void
-cb_manual_position_changed (GtkToggleButton *button, ObjectPrefState *state)
+update_select_state (ObjectPrefState *state)
 {
+	if (state->position_select_combo) {
+		int index = gog_object_get_position_flags (state->gobj, GOG_POSITION_MANUAL) == 0 ? 0 : 1;
+		
+		gtk_combo_box_set_active (GTK_COMBO_BOX (state->position_select_combo), index); 
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (state->position_notebook), index);
+	}
+}
+
+static void
+cb_manual_position_changed (GtkComboBox *combo, ObjectPrefState *state)
+{
+	int index = gtk_combo_box_get_active (combo);
+
 	gog_object_set_position_flags (state->gobj, 
-		gtk_toggle_button_get_active (button) ? GOG_POSITION_MANUAL : 0, 
+		index != 0 ? GOG_POSITION_MANUAL : 0, 
 		GOG_POSITION_MANUAL);
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (state->position_notebook), index);
 }
 
 static void
@@ -416,8 +439,21 @@ cb_anchor_changed (GtkComboBox *combo, ObjectPrefState *state)
 	GogObjectPosition position = position_anchor[gtk_combo_box_get_active (combo)].flags;
 
 	gog_object_set_position_flags (state->gobj, position, GOG_POSITION_ANCHOR);
-	if (state->manual_toggle != NULL)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (state->manual_toggle), TRUE);
+}
+
+static void
+cb_update_editor (GogObject *gobj, ObjectPrefState *state)
+{
+	if (state->x_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->x_spin), gobj->manual_position.x * 100.0);
+	if (state->y_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->y_spin), gobj->manual_position.y * 100.0);
+	if (state->w_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin), gobj->manual_position.w * 100.0);
+	if (state->h_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin), gobj->manual_position.h * 100.0);
+	
+	update_select_state (state);
 }
 
 static void
@@ -450,7 +486,13 @@ gog_object_populate_editor (GogObject *gobj,
 	state = g_new (ObjectPrefState, 1);
 	state->gobj = gobj;
 	state->gui = gui;
-	state->manual_toggle = NULL;
+	state->position_select_combo = NULL;
+	state->x_spin = NULL;
+	state->y_spin = NULL;
+	state->w_spin = NULL;
+	state->h_spin = NULL;
+	state->position_notebook = glade_xml_get_widget (gui, "position_notebook");
+
 	g_object_ref (G_OBJECT (gobj));
 
 	widget_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
@@ -491,10 +533,8 @@ gog_object_populate_editor (GogObject *gobj,
 		gtk_widget_hide (w);
 	}
 
-	if (!(allowable_positions & GOG_POSITION_COMPASS)) {
-		w =glade_xml_get_widget (gui, "automatic_position_box");
-		gtk_widget_hide (w);
-	}
+	if (!(allowable_positions & GOG_POSITION_COMPASS)) 
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (state->position_notebook), 1);
 	
 	g_object_unref (G_OBJECT (widget_size_group));
 	g_object_unref (G_OBJECT (label_size_group));
@@ -553,28 +593,36 @@ gog_object_populate_editor (GogObject *gobj,
 			w = glade_xml_get_widget (gui, "manual_sizes");
 			gtk_widget_hide (w);
 		}
-	}
+	} else 
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (state->position_notebook), 0);
 
 	g_object_unref (G_OBJECT (widget_size_group));
 	g_object_unref (G_OBJECT (label_size_group));
 
-	w = glade_xml_get_widget (gui, "manual_position_button");
 	if ((allowable_positions & GOG_POSITION_MANUAL) &&
 	    ((allowable_positions & (GOG_POSITION_COMPASS | GOG_POSITION_ALIGNMENT)) ||
 	     (allowable_positions & GOG_POSITION_SPECIAL))) {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), 
-				       gog_object_get_position_flags (gobj, GOG_POSITION_MANUAL) != 0);
-		g_signal_connect (G_OBJECT (w), "toggled", G_CALLBACK (cb_manual_position_changed), state);
-		state->manual_toggle = w;
+		state->position_select_combo = glade_xml_get_widget (gui, "position_select_combo");
+
+		update_select_state (state);
+
+		g_signal_connect (G_OBJECT (state->position_select_combo),
+				  "changed", G_CALLBACK (cb_manual_position_changed), state);
 	} else {
+		w = glade_xml_get_widget (gui, "position_select_box");
 		gtk_widget_hide (w);
 	}
 
+	state->update_editor_handler = g_signal_connect (G_OBJECT (gobj), 
+							 "update-editor", 
+							 G_CALLBACK (cb_update_editor), state);
+
 	w = glade_xml_get_widget (gui, "gog_object_prefs");
 	g_object_set_data_full (G_OBJECT (w), "state", state, 
 				(GDestroyNotify) object_pref_state_free);  
 	gog_editor_add_page (editor, w, _("Position"));
 }
+#endif
 
 static void
 gog_object_base_init (GogObjectClass *klass)
@@ -602,7 +650,9 @@ gog_object_class_init (GObjectClass *klass)
 	klass->get_property	= gog_object_get_property;
 
 	gog_klass->parent_changed  = gog_object_parent_changed;
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor = gog_object_populate_editor;
+#endif
 
 	gog_klass->can_manual_size = FALSE;
 	gog_klass->use_parent_as_proxy = FALSE;
@@ -674,6 +724,13 @@ gog_object_class_init (GObjectClass *klass)
 		NULL, NULL,
 		g_cclosure_marshal_VOID__BOOLEAN,
 		G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+	gog_object_signals [UPDATE_EDITOR] = g_signal_new ("update-editor",
+		G_TYPE_FROM_CLASS (klass),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GogObjectClass, update_editor),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 }
 
 static void
@@ -747,7 +804,7 @@ gog_object_generate_name (GogObject *obj)
 unsigned
 gog_object_get_id (GogObject const *obj)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, 0);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), 0);
 	g_return_val_if_fail (obj != 0, 0);
 
 	return obj->id;
@@ -782,7 +839,7 @@ gog_object_set_id (GogObject *obj, unsigned id)
 	GSList *ptr;
 	GogObject *child;
 
-	g_return_if_fail (GOG_OBJECT (obj) != NULL);
+	g_return_if_fail (IS_GOG_OBJECT (obj));
 
 	if (id == 0)
 		return gog_object_generate_id (obj);
@@ -822,11 +879,13 @@ dataset_dup (GogDataset const *src, GogDataset *dst)
 
 /**
  * gog_object_dup :
- * @src : #GogObject
- * @new_parent : #GogObject the parent tree for the object (can be NULL)
- * @datadup : a function to duplicate the data (a default one is used if NULL) 
+ * @src: #GogObject
+ * @new_parent: #GogObject the parent tree for the object (can be NULL)
+ * @datadup: a function to duplicate the data (a default one is used if NULL) 
  *
  * Create a deep copy of @obj using @new_parent as its parent.
+ *
+ * Returns: FIXME
  **/
 
 GogObject *
@@ -879,28 +938,28 @@ gog_object_dup (GogObject const *src, GogObject *new_parent, GogDataDuplicator d
  * gog_object_get_parent :
  * @obj : a #GogObject
  *
- * Returns @obj's parent, potentially NULL if it has not been added to a
+ * Returns: @obj's parent, potentially NULL if it has not been added to a
  * heirarchy yet.  does not change ref-count in any way.
  **/
 GogObject *
 gog_object_get_parent (GogObject const *obj)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), NULL);
 	return obj->parent;
 }
 
 /**
  * gog_object_get_parent_typed :
- * @obj : a #GogObject
- * @type : a #GType
+ * @obj: a #GogObject
+ * @t: a #GType
  *
- * Returns @obj's parent of type @type, potentially NULL if it has not been
- * added to a heirarchy yet or none of the parents are of type @type.
+ * Returns: @obj's parent of type @type, potentially NULL if it has not been
+ * added to a hierarchy yet or none of the parents are of type @type.
  **/
 GogObject *
 gog_object_get_parent_typed (GogObject const *obj, GType t)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), NULL);
 
 	for (; obj != NULL ; obj = obj->parent)
 		if (G_TYPE_CHECK_INSTANCE_TYPE (obj, t))
@@ -912,12 +971,12 @@ gog_object_get_parent_typed (GogObject const *obj, GType t)
  * gog_object_get_graph :
  * @obj : const * #GogObject
  *
- * Returns the parent graph.
+ * Returns: the parent graph.
  **/
 GogGraph *
 gog_object_get_graph (GogObject const *obj)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), NULL);
 
 	for (; obj != NULL ; obj = obj->parent)
 		if (IS_GOG_GRAPH (obj))
@@ -942,7 +1001,7 @@ gog_object_get_theme (GogObject const *obj)
 char const *
 gog_object_get_name (GogObject const *obj)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), NULL);
 	return (obj->user_name != NULL && *obj->user_name != '\0') ? obj->user_name : obj->auto_name;
 }
 
@@ -961,7 +1020,7 @@ gog_object_set_name (GogObject *obj, char *name, GError **err)
 {
 	GogObject *tmp;
 
-	g_return_if_fail (GOG_OBJECT (obj) != NULL);
+	g_return_if_fail (IS_GOG_OBJECT (obj));
 
 	if (obj->user_name == name)
 		return;
@@ -988,7 +1047,7 @@ gog_object_get_children (GogObject const *obj, GogObjectRole const *filter)
 {
 	GSList *ptr, *res = NULL;
 
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), NULL);
 
 	if (filter == NULL)
 		return g_slist_copy (obj->children);
@@ -1005,7 +1064,8 @@ gog_object_get_children (GogObject const *obj, GogObjectRole const *filter)
  * @role : a #GogObjectRole to use as a filter
  *
  * A convenience routine to handle a unique child
- * Returns NULL and spews an error if there is more than one.
+ *
+ * Returns: NULL and spews an error if there is more than one.
  **/
 GogObject *
 gog_object_get_child_by_role (GogObject const *obj, GogObjectRole const *role)
@@ -1019,6 +1079,22 @@ gog_object_get_child_by_role (GogObject const *obj, GogObjectRole const *role)
 	return res;
 }
 
+/**
+ *
+ * gog_object_get_child_by_name :
+ * @obj : a #GogObject
+ * @role : a #char to use as a filter
+ *
+ * A convenience routine to handle a unique child
+ * Returns NULL and spews an error if there is more than one.
+ **/
+GogObject *
+gog_object_get_child_by_name (GogObject const *obj, char const *name)
+{
+	return gog_object_get_child_by_role (obj,
+		gog_object_find_role_by_name (obj, name));
+}
+
 /**
  * gog_object_is_deletable :
  * @obj : a #GogObject
@@ -1028,7 +1104,7 @@ gog_object_get_child_by_role (GogObject const *obj, GogObjectRole const *role)
 gboolean
 gog_object_is_deletable (GogObject const *obj)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), FALSE);
 
 	if (IS_GOG_GRAPH (obj))
 		return FALSE;
@@ -1089,10 +1165,9 @@ gog_role_cmp_full (GogObjectRole const *a, GogObjectRole const *b)
 
 /**
  * gog_object_possible_additions :
- * @parent : a #GogObject
- *
- * returns a list of GogObjectRoles that could be added
+ * @parent: a #GogObject
  *
+ * Returns: a list of GogObjectRoles that could be added.
  * The resulting list needs to be freed
  **/
 GSList *
@@ -1129,7 +1204,7 @@ gog_object_can_reorder (GogObject const *obj, gboolean *inc_ok, gboolean *dec_ok
 	GogObject const *parent;
 	GSList *ptr;
 
-	g_return_if_fail (GOG_OBJECT (obj) != NULL);
+	g_return_if_fail (IS_GOG_OBJECT (obj));
 
 	if (inc_ok != NULL)
 		*inc_ok = FALSE;
@@ -1169,7 +1244,7 @@ gog_object_can_reorder (GogObject const *obj, gboolean *inc_ok, gboolean *dec_ok
  * @inc :
  * @goto_max :
  *
- * Returns the object just before @obj in the new ordering.
+ * Returns: the object just before @obj in the new ordering.
  **/
 GogObject *
 gog_object_reorder (GogObject const *obj, gboolean inc, gboolean goto_max)
@@ -1177,7 +1252,7 @@ gog_object_reorder (GogObject const *obj, gboolean inc, gboolean goto_max)
 	GogObject *parent, *obj_follows;
 	GSList **ptr, *tmp;
 
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), NULL);
 
 	if (obj->parent == NULL || gog_object_get_graph (obj) == NULL)
 		return NULL;
@@ -1223,16 +1298,20 @@ gog_object_reorder (GogObject const *obj, gboolean inc, gboolean goto_max)
 
 /**
  * gog_object_get_editor :
- * @obj   : #GogObject
- * @dalloc : #GogDataAllocator
- * @cc     : #GOCmdContext
- *
+ * @obj: a #GogObject
+ * @dalloc: a #GogDataAllocator
+ * @cc: a #GOCmdContext
+ * 
+ * Builds an object property editor, by calling GogObject::populate_editor
+ * virtual functions.
+ * 
+ * return value: a #GtkNotebook widget
  **/
-
 gpointer
 gog_object_get_editor (GogObject *obj, GogDataAllocator *dalloc,
 		       GOCmdContext *cc)
 {
+#ifdef GOFFICE_WITH_GTK
 	GtkWidget *notebook;
 	GogEditor *editor;
 	GogObjectClass *klass = GOG_OBJECT_GET_CLASS (obj);
@@ -1252,12 +1331,19 @@ gog_object_get_editor (GogObject *obj, GogDataAllocator *dalloc,
 	gog_editor_free (editor);
 
 	return notebook;
+#else
+	return NULL;
+#endif
 }
 
 /**
  * gog_object_new_view :
- * @obj : a #GogObject
- * @data :
+ * @obj: a #GogObject
+ * @parent: parent view
+ *
+ * Creates a new #GogView associated to @obj, and sets its parent to @parent.
+ *
+ * return value: a new #GogView
  **/
 GogView *
 gog_object_new_view (GogObject const *obj, GogView *parent)
@@ -1339,6 +1425,20 @@ gog_object_emit_changed (GogObject *obj, gboolean resize)
 		gog_object_signals [CHANGED], 0, resize);
 }
 
+/**
+ * gog_object_request_editor_update :
+ * @obj: #GogObject
+ *
+ * Emits a update-editor signal. This signal should be used by object editors
+ * in order to refresh their states.
+ **/
+void
+gog_object_request_editor_update (GogObject *obj)
+{
+	g_signal_emit (G_OBJECT (obj),
+		gog_object_signals [UPDATE_EDITOR], 0);
+}
+
 /******************************************************************************/
 
 /**
@@ -1380,16 +1480,16 @@ gog_object_clear_parent (GogObject *obj)
 
 /**
  * gog_object_set_parent :
- * @child  : #GogObject.
- * @parent : #GogObject.
- * @id : optionally %NULL.
- * @role : a static string that can be sent to @parent::add
+ * @child: #GogObject.
+ * @parent: #GogObject.
+ * @id: optionally %NULL.
+ * @role: a static string that can be sent to @parent::add
  *
  * Absorbs a ref to @child
  **/
 gboolean
 gog_object_set_parent (GogObject *child, GogObject *parent,
-		       GogObjectRole const *role, unsigned id)
+		       GogObjectRole const *role, unsigned int id)
 {
 	GogObjectClass *klass;
 	GSList **step;
@@ -1457,7 +1557,7 @@ gog_object_add_by_role (GogObject *parent, GogObjectRole const *role, GogObject
  * @role :
  * @child : optionally null #GogObject
  *
- * Returns a newly created child of @parent in @role.  If @child is provided,
+ * Returns: a newly created child of @parent in @role.  If @child is provided,
  * it is assumed to be an unaffiliated object that will be assigned in @role.
  * On failure return NULL.
  **/
@@ -1465,6 +1565,7 @@ GogObject *
 gog_object_add_by_name (GogObject *parent,
 			char const *role, GogObject *child)
 {
+	g_return_val_if_fail (IS_GOG_OBJECT (parent), NULL);
 	return gog_object_add_by_role (parent,
 		gog_object_find_role_by_name (parent, role), child);
 }
@@ -1479,23 +1580,24 @@ gog_object_add_by_name (GogObject *parent,
 GogObjectPosition
 gog_object_get_position_flags (GogObject const *obj, GogObjectPosition mask)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, GOG_POSITION_SPECIAL & mask);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), GOG_POSITION_SPECIAL & mask);
 	return obj->position & mask;
 }
 
 /**
  * gog_object_set_position_flags :
- * @obj : #GogObject
- * @flags : #GogObjectPosition
- * @mask : #GogObjectPosition
+ * @obj: #GogObject
+ * @flags: #GogObjectPosition
+ * @mask: #GogObjectPosition
  *
  * Attempts to set the position flags of @obj to @flags.
- * Returns TRUE the new flags are permitted.
+ *
+ * Returns: TRUE the new flags are permitted.
  **/
 gboolean
 gog_object_set_position_flags (GogObject *obj, GogObjectPosition flags, GogObjectPosition mask)
 {
-	g_return_val_if_fail (GOG_OBJECT (obj) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_OBJECT (obj), FALSE);
 
 	if (obj->role == NULL)
 		return FALSE;
@@ -1515,10 +1617,11 @@ gog_object_set_position_flags (GogObject *obj, GogObjectPosition flags, GogObjec
 }
 
 /**
- * gog_object_get_manual_position:
- * @obj : #GogObject
+ * gog_object_get_manual_position :
+ * @obj: #GogObject
+ * @pos: #GogViewAllocation
  *
- * returns manual position of this object, in points.
+ * FIXME
  **/
 void
 gog_object_get_manual_position (GogObject *gobj, GogViewAllocation *pos)
@@ -1557,8 +1660,8 @@ gog_object_set_manual_position (GogObject *gobj, GogViewAllocation const *pos)
  * @parent_allocation : #GogViewAllocation
  * @requisition : #GogViewRequisition
  *
- * Returns manual allocation of a GogObject given its parent allocation
- * and its size request.
+ * Returns: manual allocation of a GogObject given its parent allocation and
+ * its size request.
  **/
 GogViewAllocation
 gog_object_get_manual_allocation (GogObject *gobj,
