@@ -29,7 +29,7 @@
 #include <goffice/utils/go-marker.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
 typedef void (*GogThemeStyleMap) (GogStyle *style, unsigned ind);
@@ -139,7 +139,7 @@ static GogThemeElement *
 gog_theme_find_element (GogTheme *theme, GogObject *obj)
 {
 	GogThemeElement *elem = NULL;
-	GObjectClass *klass;
+	GObjectClass *klass = NULL;	/* make gcc happy */
 	char const *name;
 
 	if (theme == NULL)
@@ -213,11 +213,11 @@ gog_theme_find_element (GogTheme *theme, GogObject *obj)
 
 /**
  * gog_theme_fillin_style :
- * @theme : #GogTheme
- * @style : #GogStyle to initialize
- * @obj : #GogObject The object associated with @style
- * @ind : an optional index
- * @complete_overwrite : boolean
+ * @theme: #GogTheme
+ * @style: #GogStyle to initialize
+ * @obj: #GogObject The object associated with @style
+ * @ind: an optional index
+ * @complete_overwrite: boolean
  *
  * Fill in the auto aspects of @style based on @theme 's element for objects of
  * type/role similar to @obj with index @ind.  If @complete_overwrite is used,
@@ -244,7 +244,7 @@ gog_theme_fillin_style (GogTheme *theme, GogStyle *style,
 void
 gog_theme_register (GogTheme *theme, gboolean is_default)
 {
-	g_return_if_fail (GOG_THEME (theme) != NULL);
+	g_return_if_fail (IS_GOG_THEME (theme));
 
 	if (is_default) {
 		g_object_ref (theme);
@@ -329,7 +329,7 @@ gog_theme_lookup (char const *name)
 char const *
 gog_theme_get_name (GogTheme const *theme)
 {
-	g_return_val_if_fail (GOG_THEME (theme) != NULL, "");
+	g_return_val_if_fail (IS_GOG_THEME (theme), "");
 	return theme->name;
 }
 
@@ -542,15 +542,7 @@ gog_themes_init	(void)
 	style->line.width = 1;
 	style->line.color = RGBA_BLACK;
 	gog_theme_add_element (theme, style,
-		NULL, "GogRegCurve", NULL);
-
-	/* regression curves */
-	style = gog_style_new ();
-	style->line.dash_type = GO_LINE_SOLID;
-	style->line.width = 1;
-	style->line.color = RGBA_BLACK;
-	gog_theme_add_element (theme, style,
-		NULL, "GogRegCurve", NULL);
+		NULL, "GogTrendLine", NULL);
 
 	/* regression equations */
 	style = gog_style_new ();
@@ -558,7 +550,7 @@ gog_themes_init	(void)
 	style->outline.width = 0; /* hairline */
 	style->outline.color = RGBA_BLACK;
 	style->fill.type = GOG_FILL_STYLE_PATTERN;
-	go_pattern_set_solid (&style->fill.pattern, 0);
+	go_pattern_set_solid (&style->fill.pattern, RGBA_WHITE);
 	gog_theme_add_element (theme, style,
 		NULL, "GogRegEqn", NULL);
 
@@ -655,19 +647,21 @@ gog_themes_init	(void)
 	go_pattern_set_solid (&style->fill.pattern, RGBA_WHITE);
 	gog_theme_add_element (theme, style, NULL, "GogLabel", NULL);
 
-	/* regression curves */
+	/* trend lines */
 	style = gog_style_new ();
 	style->line.dash_type = GO_LINE_SOLID;
 	style->line.width = 1;
 	style->line.color = RGBA_BLACK;
 	gog_theme_add_element (theme, style,
-		NULL, "GogRegCurve", NULL);
+		NULL, "GogTrendLine", NULL);
 
 	/* regression equations */
 	style = gog_style_new ();
 	style->outline.dash_type = GO_LINE_SOLID;
 	style->outline.width = 0; /* hairline */
 	style->outline.color = RGBA_BLACK;
+	style->fill.type = GOG_FILL_STYLE_PATTERN;
+	go_pattern_set_solid (&style->fill.pattern, RGBA_WHITE);
 	gog_theme_add_element (theme, style,
 		NULL, "GogRegEqn", NULL);
 
