@@ -81,12 +81,17 @@ make_icon (GtkAction *a, const char *stock_id, GtkWidget *tool)
 {
 	GtkIconSize size;
 
-	if (tool->parent)
-		size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (tool->parent));
-	else {
-		GtkSettings *settings = gtk_widget_get_settings (tool);
-		g_object_get (settings, "gtk-toolbar-icon-size", &size, NULL);
-	}
+	if (stock_id == NULL)
+		return NULL;
+	if (IS_GO_TOOL_COMBO_PIXMAPS (tool)) {
+		if (tool->parent)
+			size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (tool->parent));
+		else {
+			GtkSettings *settings = gtk_widget_get_settings (tool);
+			g_object_get (settings, "gtk-toolbar-icon-size", &size, NULL);
+		}
+	} else
+		size = GTK_ICON_SIZE_MENU;
 
 	return gtk_widget_render_icon (tool, stock_id, size,
 				       "GOActionComboPixmaps");
@@ -103,11 +108,13 @@ go_action_combo_pixmaps_connect_proxy (GtkAction *a, GtkWidget *proxy)
 		GOActionComboPixmaps *paction = (GOActionComboPixmaps *)a;
 		const char *stock_id = paction->elements[0].stock_id;
 		GdkPixbuf *icon = make_icon (a, stock_id, proxy);
-		GtkWidget *image = gtk_image_new_from_pixbuf (icon);
-		g_object_unref (icon);
-		gtk_widget_show (image);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (proxy),
-					       image);
+		if (icon) {
+			GtkWidget *image = gtk_image_new_from_pixbuf (icon);
+			g_object_unref (icon);
+			gtk_widget_show (image);
+			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (proxy),
+							   image);
+		}
 	}
 }
 
