@@ -28,16 +28,9 @@
 #include "go-combo-box.h"
 #include <goffice/utils/go-marshalers.h>
 
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtktearoffmenuitem.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkarrow.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkframe.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <gsf/gsf-impl-utils.h>
 
@@ -81,29 +74,36 @@ static void go_combo_set_tearoff_state (GOComboBox *combo, gboolean torn_off);
  */
 static void
 go_combo_popup_reparent (GtkWidget *popup,
-			  GtkWidget *new_parent,
-			  gboolean unrealize)
+			 GtkWidget *new_parent,
+			 gboolean unrealize)
 {
-	GtkObject *object = GTK_OBJECT (popup);
-	gboolean was_floating = GTK_OBJECT_FLOATING (object);
-
-	g_object_ref (object);
-	gtk_object_sink (object);
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+	gboolean was_floating = g_object_is_floating (popup);
+	g_object_ref_sink (popup);
+#else
+	gboolean was_floating = GTK_OBJECT_FLOATING (popup);
+	g_object_ref (popup);
+	gtk_object_sink (GTK_OBJECT (popup));
+#endif
 
 	if (unrealize) {
-		g_object_ref (object);
+		g_object_ref (popup);
 		gtk_container_remove (GTK_CONTAINER (popup->parent), popup);
 		gtk_container_add (GTK_CONTAINER (new_parent), popup);
-		g_object_unref (object);
+		g_object_unref (popup);
 	}
 	else
 		gtk_widget_reparent (GTK_WIDGET (popup), new_parent);
 	gtk_widget_set_size_request (new_parent, -1, -1);
 
-	if (was_floating)
-		GTK_OBJECT_SET_FLAGS (object, GTK_FLOATING);
-	else
-		g_object_unref (object);
+	if (was_floating) {
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+		g_object_force_floating (G_OBJECT (popup));
+#else
+		GTK_OBJECT_SET_FLAGS (GTK_OBJECT (popup), GTK_FLOATING);
+#endif
+	} else
+		g_object_unref (popup);
 }
 
 static void
@@ -311,12 +311,16 @@ go_combo_popup_tear_off (GOComboBox *combo, gboolean set_position)
 
 	if (!combo->priv->tearoff_window) {
 		GtkWidget *tearoff;
-		const gchar *title;
+		gchar const *title;
 
 		/* FIXME: made this a toplevel, not a dialog ! */
 		tearoff = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_widget_ref (tearoff);
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+		g_object_ref_sink (tearoff);
+#else
+		g_object_ref (tearoff);
 		gtk_object_sink (GTK_OBJECT (tearoff));
+#endif
 		combo->priv->tearoff_window = tearoff;
 		gtk_widget_set_app_paintable (tearoff, TRUE);
 		g_signal_connect (tearoff, "key_press_event",
@@ -361,9 +365,9 @@ go_combo_popup_tear_off (GOComboBox *combo, gboolean set_position)
 
 /**
  * go_combo_box_popup_hide:
- * @combo_box:  Combo box
+ * @combo: a #GOComboBox
  *
- * Hide popup, but not when it is torn off.
+ * Hides popup, but not when it is torn off.
  * This is the external interface - for subclasses and apps which expect a
  * regular combo which doesn't do tearoffs.
  */
@@ -541,8 +545,12 @@ go_combo_box_init (GOComboBox *combo_box)
 	 */
 
 	combo_box->priv->toplevel = gtk_window_new (GTK_WINDOW_POPUP);
-	gtk_widget_ref (combo_box->priv->toplevel);
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+	g_object_ref_sink (combo_box->priv->toplevel);
+#else
+	g_object_ref (combo_box->priv->toplevel);
 	gtk_object_sink (GTK_OBJECT (combo_box->priv->toplevel));
+#endif
 	g_object_set (G_OBJECT (combo_box->priv->toplevel),
 		"allow-shrink",	FALSE,
 		"allow-grow",	TRUE,
@@ -578,11 +586,11 @@ GSF_CLASS (GOComboBox, go_combo_box,
 
 /**
  * go_combo_box_set_display:
- * @combo_box: the Combo Box to modify
+ * @combo: the #GOComboBox to modify
  * @display_widget: The widget to be displayed
 
  * Sets the displayed widget for the @combo_box to be @display_widget
- */
+ **/
 /* protected */ void
 go_combo_box_set_display (GOComboBox *combo_box, GtkWidget *display_widget)
 {
