@@ -24,6 +24,7 @@
 #include "gog-1.5d.h"
 #include <goffice/graph/gog-series-lines.h>
 #include <goffice/graph/gog-view.h>
+#include <goffice/graph/gog-chart.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-axis.h>
@@ -32,7 +33,7 @@
 #include <goffice/utils/go-marker.h>
 #include <goffice/utils/go-math.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 
 struct _GogLinePlot {
@@ -47,11 +48,60 @@ enum {
 	GOG_LINE_PROP_DEFAULT_STYLE_HAS_MARKERS
 };
 
-typedef GogSeries1_5d		GogLineSeries;
+/*****************************************************************************/
+
+typedef GogView		GogLineSeriesView;
+typedef GogViewClass	GogLineSeriesViewClass;
+
+#define GOG_LINE_SERIES_VIEW_TYPE	(gog_line_series_view_get_type ())
+#define GOG_LINE_SERIES_VIEW(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_LINE_SERIES_VIEW_TYPE, GogLineSeriesView))
+#define IS_GOG_LINE_SERIES_VIEW(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_LINE_SERIES_VIEW_TYPE))
+
+static void
+gog_line_series_view_render (GogView *view, GogViewAllocation const *bbox)
+{
+	GSList *ptr;
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next)
+		gog_view_render	(ptr->data, bbox);
+}
+
+static void
+gog_line_series_view_size_allocate (GogView *view, GogViewAllocation const *allocation)
+{
+	GSList *ptr;
+
+	for (ptr = view->children; ptr != NULL; ptr = ptr->next)
+		gog_view_size_allocate (GOG_VIEW (ptr->data), allocation);
+}
+
+static void
+gog_line_series_view_class_init (GogLineSeriesViewClass *gview_klass)
+{
+	GogViewClass *view_klass = GOG_VIEW_CLASS (gview_klass);
+	view_klass->render = gog_line_series_view_render;
+	view_klass->size_allocate = gog_line_series_view_size_allocate;
+	view_klass->build_toolkit = NULL;
+}
+
+GSF_DYNAMIC_CLASS (GogLineSeriesView, gog_line_series_view,
+	gog_line_series_view_class_init, NULL,
+	GOG_VIEW_TYPE)
+
+/*****************************************************************************/
+
+typedef struct {
+	GogSeries1_5d base;
+	double *x;
+} GogLineSeries;
 typedef GogSeries1_5dClass	GogLineSeriesClass;
 
 static GogStyledObjectClass *series_parent_klass;
 
+GType gog_line_series_get_type (void);
+#define GOG_LINE_SERIES_TYPE	(gog_line_series_get_type ())
+#define GOG_LINE_SERIES(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_LINE_SERIES_TYPE, GogLineSeries))
+#define IS_GOG_LINE_SERIES(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_LINE_SERIES_TYPE))
+
 static void
 gog_line_series_init_style (GogStyledObject *gso, GogStyle *style)
 {
@@ -63,6 +113,7 @@ gog_line_series_init_style (GogStyledObject *gso, GogStyle *style)
 		return;
 
 	plot = GOG_LINE_PLOT (series->plot);
+
 	if (!plot->default_style_has_markers) {
 		style->disable_theming |= GOG_STYLE_MARKER;
 		if (style->marker.auto_shape) {
@@ -74,17 +125,78 @@ gog_line_series_init_style (GogStyledObject *gso, GogStyle *style)
 }
 
 static void
-gog_line_series_class_init (GogStyledObjectClass *gso_klass)
+gog_line_series_update (GogObject *obj)
 {
+	GogLineSeries *series = GOG_LINE_SERIES (obj);
+	unsigned i, nb = series->base.base.num_elements;
+	GSList *ptr;
+	(GOG_OBJECT_CLASS (series_parent_klass))->update (obj);
+	if (nb != series->base.base.num_elements) {
+		nb = series->base.base.num_elements;
+		if (series->x) {
+			g_free (series->x);
+			series->x = NULL;
+		}
+		series->x = g_new (double, nb);
+		for (i = 0; i < nb; i++)
+			series->x[i] = i + 1;
+	}
+	/* update children */
+	for (ptr = obj->children; ptr != NULL; ptr = ptr->next)
+		if (!IS_GOG_SERIES_LINES (ptr->data))
+			gog_object_request_update (GOG_OBJECT (ptr->data));
+}
+
+static unsigned
+gog_line_series_get_xy_data (GogSeries const *series,
+					double const **x, double const **y)
+{
+	GogLineSeries *line_ser = GOG_LINE_SERIES (series);
+	*x = line_ser->x;
+	*y = go_data_vector_get_values (GO_DATA_VECTOR (series->values[1].data));
+	return series->num_elements;
+}
+
+static void
+gog_line_series_finalize (GObject *obj)
+{
+	GogLineSeries *series = GOG_LINE_SERIES (obj);
+
+	if (series->x != NULL) {
+		g_free (series->x); 
+		series->x = NULL;
+	}
+
+	G_OBJECT_CLASS (series_parent_klass)->finalize (obj);
+}
+
+static void
+gog_line_series_class_init (GogStyledObjectClass *gso_klass)
+{
+	GObjectClass *obj_klass = (GObjectClass *)gso_klass;
+	GogObjectClass *gog_klass = (GogObjectClass *)gso_klass;
+	GogSeriesClass *series_klass = (GogSeriesClass*) gso_klass;
 	series_parent_klass = g_type_class_peek_parent (gso_klass);
+	obj_klass->finalize = gog_line_series_finalize;
 	gso_klass->init_style = gog_line_series_init_style;
+	gog_klass->view_type = gog_line_series_view_get_type ();
+	gog_klass->update = gog_line_series_update;
+	series_klass->get_xy_data = gog_line_series_get_xy_data;
 }
 
-GType gog_line_series_get_type (void);
 GSF_DYNAMIC_CLASS (GogLineSeries, gog_line_series,
 	gog_line_series_class_init, NULL,
 	GOG_SERIES1_5D_TYPE)
 
+static void
+child_added_cb (GogLinePlot *plot, GogObject *obj)
+{
+	/* we only accept regression curves for not stacked plots */
+	if (IS_GOG_SERIES (obj) && plot->base.type == GOG_1_5D_NORMAL)
+		(GOG_SERIES (obj))->acceptable_children =
+					GOG_SERIES_ACCEPT_TREND_LINE;
+}
+
 static char const *
 gog_line_plot_type_name (G_GNUC_UNUSED GogObject const *item)
 {
@@ -196,6 +308,8 @@ static void
 gog_line_plot_init (GogLinePlot *plot)
 {
 	plot->default_style_has_markers = TRUE;
+	g_signal_connect (G_OBJECT (plot), "child-added",
+					G_CALLBACK (child_added_cb), NULL);
 	GOG_PLOT1_5D (plot)->support_drop_lines = TRUE;
 }
 
@@ -255,6 +369,9 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogPlot1_5d const *model = GOG_PLOT1_5D (view->model);
 	GogPlot1_5dType const type = model->type;
 	GogSeries1_5d const *series;
+	GogChart *chart = GOG_CHART (view->model->parent);
+	GogChartMap *chart_map;
+	GogViewAllocation const *area;
 	unsigned i, j, k;
 	unsigned num_elements = model->num_elements;
 	unsigned num_series = model->num_series;
@@ -270,7 +387,7 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogObjectRole const *role = NULL;
 	GogSeriesLines **lines;
 
-	double y_zero;
+	double y_zero, drop_lines_y_zero;
 	double abs_sum, sum, value;
 	gboolean is_null, is_area_plot;
 
@@ -281,20 +398,25 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 	if (num_elements <= 0 || num_series <= 0)
 		return;
 
-	x_map = gog_axis_map_new (model->base.axis[0], 
-				  view->allocation.x, view->allocation.w);
-	y_map = gog_axis_map_new (model->base.axis[1], 
-				  view->allocation.y + view->allocation.h, 
-				  -view->allocation.h);
-	
-	if (!(gog_axis_map_is_valid (x_map) &&
-	      gog_axis_map_is_valid (y_map))) {
-		gog_axis_map_free (x_map);
-		gog_axis_map_free (y_map);
+	area = gog_chart_view_get_plot_area (view->parent);
+	chart_map = gog_chart_map_new (chart, area, 
+				       GOG_PLOT (model)->axis[GOG_AXIS_X], 
+				       GOG_PLOT (model)->axis[GOG_AXIS_Y],
+				       NULL, FALSE);
+	if (!gog_chart_map_is_valid (chart_map)) {
+		gog_chart_map_free (chart_map);
 		return;
 	}
 	
-	y_zero = gog_axis_map_get_baseline (y_map);
+	x_map = gog_chart_map_get_axis_map (chart_map, 0);
+	y_map = gog_chart_map_get_axis_map (chart_map, 1);
+	
+	/* Draw drop lines from point to axis start. See comment in
+	 * GogXYPlotView::render */
+
+	gog_axis_map_get_extents (y_map, &drop_lines_y_zero, NULL); 
+	drop_lines_y_zero = gog_axis_map_to_view (y_map, drop_lines_y_zero);
+	y_zero = gog_axis_map_get_baseline (y_map); 
 
 	vals    = g_alloca (num_series * sizeof (double *));
 	error_data = g_alloca (num_series * sizeof (ErrorBarData *));
@@ -340,7 +462,7 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 			for (j = 0; j < num_elements; j++) {
 				drop_paths[i][2 * j].code = ART_MOVETO;
 				drop_paths[i][2 * j + 1].code = ART_LINETO;
-				drop_paths[i][2 * j + 1].y = y_zero; 
+				drop_paths[i][2 * j + 1].y = drop_lines_y_zero; 
 			}
 			drop_paths[i][2 * j].code = ART_END;
 		} else
@@ -375,7 +497,7 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 			k = 2 * lengths[i] - j + 1;
 
 			if (is_area_plot && (type != GOG_1_5D_NORMAL)) {
-				path[i][k].x = gog_axis_map_to_view (x_map, j - 1);
+				path[i][k].x = gog_axis_map_to_view (x_map, j);
 				path[i][k].code = ART_LINETO;
 
 				if (type == GOG_1_5D_STACKED)
@@ -390,7 +512,7 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 						 y_zero);
 			}
 
-			path[i][j].x = gog_axis_map_to_view (x_map, j - 1);
+			path[i][j].x = gog_axis_map_to_view (x_map, j);
 			if (type == GOG_1_5D_NORMAL && !is_area_plot) 
 				if (gog_axis_map_finite (y_map, vals[i][j-1])) 
 					if (j > 1 && path[i][j-1].code == ART_MOVETO_OPEN)
@@ -405,7 +527,7 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 			sum += value;
 
 			if (gog_error_bar_is_visible (errors[i])) 
-				error_data[i][j-1].x = j - 1;
+				error_data[i][j-1].x = j;
 
 			switch (type) {
 				case GOG_1_5D_NORMAL :
@@ -557,27 +679,30 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 		g_free (error_data[i]);
 	}
 
-	gog_axis_map_free (x_map);
-	gog_axis_map_free (y_map);
+	/* Now render children */
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next)
+		gog_view_render	(ptr->data, bbox);
+
+	gog_chart_map_free (chart_map);
 }
 
-static gboolean
-gog_line_view_info_at_point (GogView *view, double x, double y,
-			     GogObject const *cur_selection,
-			     GogObject **obj, char **name)
+static GogViewClass *line_view_parent_klass;
+
+static void
+gog_line_view_size_allocate (GogView *view, GogViewAllocation const *allocation)
 {
-	if (obj != NULL)
-		*obj = view->model;
-	if (name != NULL)
-		*name = g_strdup (gog_object_get_name (GOG_OBJECT (view->model)));
-	return TRUE;
+	GSList *ptr;
+	for (ptr = view->children; ptr != NULL; ptr = ptr->next)
+		gog_view_size_allocate (GOG_VIEW (ptr->data), allocation);
+	(line_view_parent_klass->size_allocate) (view, allocation);
 }
 
 static void
 gog_line_view_class_init (GogViewClass *view_klass)
 {
+	line_view_parent_klass = (GogViewClass*) g_type_class_peek_parent (view_klass);
 	view_klass->render	  = gog_line_view_render;
-	view_klass->info_at_point = gog_line_view_info_at_point;
+	view_klass->size_allocate = gog_line_view_size_allocate;
 }
 
 GSF_DYNAMIC_CLASS (GogLineView, gog_line_view,
