@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -33,7 +33,7 @@
 #include <goffice/utils/go-color.h>
 #include <goffice/app/module-plugin-defs.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 
 #include <locale.h>
@@ -198,13 +198,16 @@ extern gpointer gog_contour_plot_pref (GogContourPlot *plot, GOCmdContext *cc);
 static void
 gog_contour_plot_populate_editor (GogObject *item,
 				  GogEditor *editor,
-		    G_GNUC_UNUSED GogDataAllocator *dalloc,
-		    GOCmdContext *cc)
+				  G_GNUC_UNUSED GogDataAllocator *dalloc,
+				  GOCmdContext *cc)
 {
 	gog_editor_add_page (editor,
 			     gog_contour_plot_pref (GOG_CONTOUR_PLOT (item), cc),
 			     _("Properties"));
+
+	(GOG_OBJECT_CLASS(plot_contour_parent_klass)->populate_editor) (item, editor, dalloc, cc);
 }
+
 static void
 gog_contour_plot_clear_formats (GogContourPlot *plot)
 {
@@ -299,27 +302,9 @@ gog_contour_plot_update (GogObject *obj)
 		plot_contour_parent_klass->update (obj);
 }
 
-static GogAxisSet
-gog_contour_plot_axis_set_pref (GogPlot const *plot)
-{
-	return GOG_AXIS_SET_XY_pseudo_3d;
-}
-
-static gboolean
-gog_contour_plot_axis_set_is_valid (GogPlot const *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY_pseudo_3d;
-}
-
-static gboolean
-gog_contour_plot_axis_set_assign (GogPlot *plot, GogAxisSet type)
-{
-	return type == GOG_AXIS_SET_XY_pseudo_3d;
-}
-
 static GOData *
 gog_contour_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis, 
-				GogPlotBoundInfo * bounds)
+				  GogPlotBoundInfo * bounds)
 {
 	GogSurfaceSeries *series;
 	GogContourPlot *contour = GOG_CONTOUR_PLOT (plot);
@@ -354,12 +339,15 @@ gog_contour_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 		bounds->logical.maxima = bounds->val.maxima = max;
 		bounds->is_discrete = FALSE;
 	} else {
-		bounds->val.minima = 0.;
-		bounds->logical.minima = 0.;
+		bounds->val.minima = 1.;
+		bounds->logical.minima = 1.;
 		bounds->logical.maxima = go_nan;
 		bounds->is_discrete    = TRUE;
 		bounds->center_on_ticks = TRUE;
-		bounds->val.maxima = (axis == GOG_AXIS_X)? series->columns: series->rows;
+		bounds->val.maxima = ((axis == GOG_AXIS_Y && contour->transposed) ||
+		(axis == GOG_AXIS_X && !contour->transposed)) ?
+			series->columns:
+			series->rows;
 	}
 	return (GOData*) vec;
 }
@@ -439,11 +427,15 @@ gog_contour_plot_set_property (GObject *obj, guint param_id,
 
 	switch (param_id) {
 	case CONTOUR_PROP_TRANSPOSED :
-		plot->transposed = g_value_get_boolean (value);
-		gog_axis_bound_changed (plot->base.axis[GOG_AXIS_X], GOG_OBJECT (plot));
-		gog_axis_bound_changed (plot->base.axis[GOG_AXIS_Y], GOG_OBJECT (plot));
-		g_free (plot->plotted_data);
-		plot->plotted_data = NULL;
+		if (!plot->transposed != !g_value_get_boolean (value)) {
+			plot->transposed = g_value_get_boolean (value);
+			if (NULL != plot->base.axis[GOG_AXIS_X])
+				gog_axis_bound_changed (plot->base.axis[GOG_AXIS_X], GOG_OBJECT (plot));
+			if (NULL != plot->base.axis[GOG_AXIS_Y])
+				gog_axis_bound_changed (plot->base.axis[GOG_AXIS_Y], GOG_OBJECT (plot));
+			g_free (plot->plotted_data);
+			plot->plotted_data = NULL;
+		}
 		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
@@ -507,11 +499,9 @@ gog_contour_plot_class_init (GogContourPlotClass *klass)
 
 	/* Fill in GogPlotClass methods */
 	gog_plot_klass->desc.num_series_min = 1;
-	gog_plot_klass->desc.num_series_max = G_MAXINT;
+	gog_plot_klass->desc.num_series_max = 1;
 	gog_plot_klass->series_type = gog_surface_series_get_type();
-	gog_plot_klass->axis_set_pref = gog_contour_plot_axis_set_pref;
-	gog_plot_klass->axis_set_is_valid = gog_contour_plot_axis_set_is_valid;
-	gog_plot_klass->axis_set_assign = gog_contour_plot_axis_set_assign;
+	gog_plot_klass->axis_set = GOG_AXIS_SET_XY_pseudo_3d;
 	gog_plot_klass->axis_get_bounds	= gog_contour_plot_axis_get_bounds;
 	gog_plot_klass->foreach_elem = gog_contour_plot_foreach_elem;
 	gog_plot_klass->update_3d = gog_contour_plot_update_3d;
@@ -535,21 +525,22 @@ gog_contour_plot_init (GogContourPlot *contour)
 	plot->render_before_axes = TRUE;
 }
 
-GSF_CLASS (GogContourPlot, gog_contour_plot,
-	   gog_contour_plot_class_init, gog_contour_plot_init,
-	   GOG_PLOT_TYPE)
+GSF_DYNAMIC_CLASS (GogContourPlot, gog_contour_plot,
+	gog_contour_plot_class_init, gog_contour_plot_init,
+	GOG_PLOT_TYPE)
 
 /*****************************************************************************/
 
 typedef GogPlotView		GogContourView;
 typedef GogPlotViewClass	GogContourViewClass;
+#define CONTOUR_EPSILON 1e-10
 
 static void
 gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 {
 	GogContourPlot const *plot = GOG_CONTOUR_PLOT (view->model);
 	GogSeries const *series = GOG_SERIES (plot->base.series->data);
-	GODataVector *x_vec = 0, *y_vec = 0;
+	GODataVector *x_vec = NULL, *y_vec = NULL;
 	GogAxisMap *x_map, *y_map;
 	double zval0, zval1, zval2 = 0., zval3, t;
 	double x[4], y[4], zval[4];
@@ -632,21 +623,27 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 	g_object_unref (style);
 
 	/* clip to avoid problems with logarithmic axes */
-	gog_renderer_clip_push (rend, &(view->residual));
+	gog_renderer_push_clip (rend, 
+				gog_renderer_get_rectangle_vpath (&view->residual));
 
 	style = gog_style_new ();
-	style->interesting_fields = GOG_STYLE_FILL;
+	style->interesting_fields = GOG_STYLE_FILL | GOG_STYLE_OUTLINE;
 	style->disable_theming = GOG_STYLE_ALL;
 	style->fill.type = GOG_FILL_STYLE_PATTERN;
 	style->fill.pattern.pattern = GO_PATTERN_SOLID;
+	style->outline.dash_type = GO_LINE_SOLID;
+	style->outline.auto_dash = FALSE;
+	style->outline.auto_color = FALSE;
+	style->outline.width = 0.5;
+	style->outline.color = RGBA_BLACK;
 
 	lines = art_new (ArtVpath, lmax = 64);
 	l = 0;
 
 	for (j = 1; j < jmax; j++) {
 		if (xdiscrete) {
-			x0 = gog_axis_map_to_view (x_map, j - 1);
-			x1 = gog_axis_map_to_view (x_map, j);
+			x0 = gog_axis_map_to_view (x_map, j);
+			x1 = gog_axis_map_to_view (x_map, j + 1);
 		}else {
 			x0 = gog_axis_map_to_view (x_map, go_data_vector_get_value (x_vec, j - 1));
 			x1 = gog_axis_map_to_view (x_map, go_data_vector_get_value (x_vec, j));
@@ -654,8 +651,8 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 		
 		for (i = 1; i < imax; i++) {
 			if (ydiscrete) {
-				y0 = gog_axis_map_to_view (y_map, i - 1);
-				y1 = gog_axis_map_to_view (y_map, i);
+				y0 = gog_axis_map_to_view (y_map, i);
+				y1 = gog_axis_map_to_view (y_map, i + 1);
 			}else {
 				y0 = gog_axis_map_to_view (y_map, go_data_vector_get_value (y_vec, i - 1));
 				y1 = gog_axis_map_to_view (y_map, go_data_vector_get_value (y_vec, i));
@@ -761,6 +758,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 			} while (k != r);
 			if (zmin == zmax) {
 				/* paint everything with one color*/
+				style->outline.color = color[zmin];
 				style->fill.pattern.back = color[zmin];
 				gog_renderer_push_style (rend, style);
 				path[0].code = ART_MOVETO;
@@ -773,7 +771,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 				path[k].y = y[0];
 				path[k + 1].code = ART_END;
 				/* narrow parameter is TRUE below to avoid border effects */
-				gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+				gog_renderer_draw_polygon (rend, path, FALSE);
 				gog_renderer_pop_style (rend);
 			} else {
 				kmax = 3 - nans;
@@ -812,6 +810,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 					/* low values slices */
 					if (z[0] < zn) {
 						k = z[0];
+						style->outline.color = color[k];
 						style->fill.pattern.back = color[k];
 						k++;
 						path[0].code = ART_MOVETO;
@@ -832,11 +831,12 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 						xl[0] = lines[l].x = path[2].x = x[0] + t * (x[1] - x[0]);
 						yl[0] = lines[l++].y = path[2].y = y[0] + t * (y[1] - y[0]);
 						gog_renderer_push_style (rend, style);
-						gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+						gog_renderer_draw_polygon (rend, path, FALSE);
 						gog_renderer_pop_style (rend);
 						path[4].code = ART_LINETO;
 						path[5].code = ART_END;
 						while (k < zn) {
+							style->outline.color = color[k];
 							style->fill.pattern.back = color[k];
 							k++;
 							path[0].x = path[4].x = xl[7];
@@ -854,13 +854,14 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							xl[0] = lines[l].x = path[2].x = x[0] + t * (x[1] - x[0]);
 							yl[0] = lines[l++].y = path[2].y = y[0] + t * (y[1] - y[0]);
 							gog_renderer_push_style (rend, style);
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							gog_renderer_pop_style (rend);
 						}
 					} else
 						xl[0] = xl[7] = -1.;
 					if (z[2] < zn) {
 						k = z[2];
+						style->outline.color = color[k];
 						style->fill.pattern.back = color[k];
 						k++;
 						path[0].code = ART_MOVETO;
@@ -881,11 +882,12 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 						xl[4] = lines[l].x = path[2].x = x[2] + t * (x[3] - x[2]);
 						yl[4] = lines[l++].y = path[2].y = y[2] + t * (y[3] - y[2]);
 						gog_renderer_push_style (rend, style);
-						gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+						gog_renderer_draw_polygon (rend, path, FALSE);
 						gog_renderer_pop_style (rend);
 						path[4].code = ART_LINETO;
 						path[5].code = ART_END;
 						while (k < zn) {
+							style->outline.color = color[k];
 							style->fill.pattern.back = color[k];
 							k++;
 							path[0].x = path[4].x = xl[3];
@@ -903,7 +905,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							xl[4] = lines[l].x = path[2].x = x[2] + t * (x[3] - x[2]);
 							yl[4] = lines[l++].y = path[2].y = y[2] + t * (y[3] - y[2]);
 							gog_renderer_push_style (rend, style);
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							gog_renderer_pop_style (rend);
 						}
 					} else
@@ -930,9 +932,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 						t = (k - zval[1]) / (zval[2] - zval[1]);
 						xl[2] = lines[l].x = path[2].x = x[1] + t * (x[2] - x[1]);
 						yl[2] = lines[l++].y = path[2].y = y[1] + t * (y[2] - y[1]);
+						style->outline.color = color[k];
 						style->fill.pattern.back = color[k];
 						gog_renderer_push_style (rend, style);
-						gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+						gog_renderer_draw_polygon (rend, path, FALSE);
 						gog_renderer_pop_style (rend);
 						path[4].code = ART_LINETO;
 						path[5].code = ART_END;
@@ -952,9 +955,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							t = (k - zval[1]) / (zval[2] - zval[1]);
 							xl[2] = lines[l].x = path[2].x = x[1] + t * (x[2] - x[1]);
 							yl[2] = lines[l++].y = path[2].y = y[1] + t * (y[2] - y[1]);
+							style->outline.color = color[k];
 							style->fill.pattern.back = color[k];
 							gog_renderer_push_style (rend, style);
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							gog_renderer_pop_style (rend);
 							k--;
 						}
@@ -981,9 +985,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 						t = (k - zval[3]) / (zval[0] - zval[3]);
 						xl[6] = lines[l].x = path[2].x = x[3] + t * (x[0] - x[3]);
 						yl[6] = lines[l++].y = path[2].y = y[3] + t * (y[0] - y[3]);
+						style->outline.color = color[k];
 						style->fill.pattern.back = color[k];
 						gog_renderer_push_style (rend, style);
-						gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+						gog_renderer_draw_polygon (rend, path, FALSE);
 						gog_renderer_pop_style (rend);
 						path[4].code = ART_LINETO;
 						path[5].code = ART_END;
@@ -1003,9 +1008,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							t = (k - zval[3]) / (zval[0] - zval[3]);
 							xl[6] = lines[l].x = path[2].x = x[3] + t * (x[0] - x[3]);
 							yl[6] = lines[l++].y = path[2].y = y[3] + t * (y[0] - y[3]);
+							style->outline.color = color[k];
 							style->fill.pattern.back = color[k];
 							gog_renderer_push_style (rend, style);
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							gog_renderer_pop_style (rend);
 							k--;
 						}
@@ -1066,9 +1072,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							path[2].y = yc;
 							path[3].x = xb[0];
 							path[3].y = yb[0];
+							style->outline.color = color[zn];
 							style->fill.pattern.back = color[zn];
 							gog_renderer_push_style (rend, style);
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							if (xl[2] < 0.) {
 								path[4].x = path[0].x = x[2];
 								path[4].y = path[0].y = y[2];
@@ -1085,7 +1092,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							path[1].y = yb[1];
 							path[3].x = xb[2];
 							path[3].y = yb[2];
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							gog_renderer_pop_style (rend);
 							if (xl[2] < 0.) {
 								path[4].x = path[0].x = x[1];
@@ -1103,9 +1110,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							path[1].y = yb[0];
 							path[3].x = xb[1];
 							path[3].y = yb[1];
+							style->outline.color = color[zx];
 							style->fill.pattern.back = color[zx];
 							gog_renderer_push_style (rend, style);
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							if (xl[6] < 0.) {
 								path[4].x = path[0].x = x[3];
 								path[4].y = path[0].y = y[3];
@@ -1122,7 +1130,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							path[1].y = yb[2];
 							path[3].x = xb[3];
 							path[3].y = yb[3];
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							gog_renderer_pop_style (rend);
 						} else {
 							if (up) {
@@ -1154,9 +1162,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 								t = (zx - zval[1]) / (zval[2] - zval[1]);
 								xl[2] = lines[l].x = path[2].x = x[1] + t * (x[2] - x[1]);
 								yl[2] = lines[l++].y = path[2].y = y[1] + t * (y[2] - y[1]);
+								style->outline.color = color[zx];
 								style->fill.pattern.back = color[zx];
 								gog_renderer_push_style (rend, style);
-								gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+								gog_renderer_draw_polygon (rend, path, FALSE);
 								if (xl[5] < 0.) {
 									path[4].code = ART_END;
 									path[0].x = path[3].x = x[3];
@@ -1179,7 +1188,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 								t = (zx - zval[3]) / (zval[0] - zval[3]);
 								xl[6] = lines[l].x = path[2].x = x[3] + t * (x[0] - x[3]);
 								yl[6] = lines[l++].y = path[2].y = y[3] + t * (y[0] - y[3]);
-								gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+								gog_renderer_draw_polygon (rend, path, FALSE);
 								gog_renderer_pop_style (rend);
 							} else {
 								/* saddle point is in the upper slice */
@@ -1209,10 +1218,11 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 								t = (k - zval[0]) / (zval[1] - zval[0]);
 								xl[0] = lines[l].x = path[2].x = x[0] + t * (x[1] - x[0]);
 								yl[0] = lines[l++].y = path[2].y = y[0] + t * (y[1] - y[0]);
+								style->outline.color = color[zn];
 								style->fill.pattern.back = color[zn];
 								gog_renderer_push_style (rend, style);
-								gog_renderer_draw_polygon (rend, path, TRUE, NULL);
-								if (xl[2] < 0.) {
+								gog_renderer_draw_polygon (rend, path, FALSE);
+								if (xl[4] < 0.) {
 									path[4].code = ART_END;
 									path[0].x = path[3].x = x[2];
 									path[0].y = path[3].y = y[2];
@@ -1232,7 +1242,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 								t = (k - zval[2]) / (zval[3] - zval[2]);
 								xl[4] = lines[l].x = path[2].x = x[2] + t * (x[3] - x[2]);
 								yl[4] = lines[l++].y = path[2].y = y[2] + t * (y[3] - y[2]);
-								gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+								gog_renderer_draw_polygon (rend, path, FALSE);
 								gog_renderer_pop_style (rend);
 								zn = zx;
 							}
@@ -1257,9 +1267,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							path[k].x = path[0].x;
 							path[k++].y = path[0].y;
 							path[k].code = ART_END;
+							style->outline.color = color[zn];
 							style->fill.pattern.back = color[zn];
 							gog_renderer_push_style (rend, style);
-							gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+							gog_renderer_draw_polygon (rend, path, FALSE);
 							gog_renderer_pop_style (rend);
 						}
 					} else {
@@ -1283,9 +1294,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 						path[k].x = path[0].x;
 						path[k++].y = path[0].y;
 						path[k].code = ART_END;
+						style->outline.color = color[zx];
 						style->fill.pattern.back = color[zx];
 						gog_renderer_push_style (rend, style);
-						gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+						gog_renderer_draw_polygon (rend, path, FALSE);
 						gog_renderer_pop_style (rend);
 					}
 				} else {
@@ -1300,59 +1312,87 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 					s = 0;
 					r = kmax;
 					while (zmin < zmax) {
+						style->outline.color = color[zmin];
 						style->fill.pattern.back = color[zmin];
 						gog_renderer_push_style (rend, style);
 						while (z[k] <= zmin && k < kmax) {
-							path[p].code = ART_LINETO;
-							path[p].x = x[k];
-							path[p++].y = y[k++];
+							if (fabs (path[p-1].x - x[k]) > CONTOUR_EPSILON ||
+								fabs (path[p-1].y - y[k]) > CONTOUR_EPSILON) {
+								path[p].code = ART_LINETO;
+								path[p].x = x[k];
+								path[p++].y = y[k++];
+							} else
+								k++;	
 						}
 						while (z[r] <= zmin && r > 0)
 							r--;
-						
 						zmin++;
 						t = (zmin - zval[k - 1]) / (zval[k] - zval[k - 1]);
 						path[p].code = ART_LINETO;
 						lines[l].code = ART_MOVETO_OPEN;
 						lines[l].x = path[p].x = x[k - 1] + t * (x[k] - x[k - 1]);
-						lines[l++].y = path[p++].y = y[k - 1] + t * (y[k] - y[k - 1]);
+						lines[l++].y = path[p].y = y[k - 1] + t * (y[k] - y[k - 1]);
+						if (fabs (path[p-1].x - path[p].x) > CONTOUR_EPSILON ||
+							fabs (path[p-1].y - path[p].y) > CONTOUR_EPSILON)
+							p++;
 						path[p].code = ART_LINETO;
 						lines[l].code = ART_LINETO;
 						if (r < kmax) {
 							t = (zmin - zval[r]) / (zval[r + 1] - zval[r]);
 							lines[l].x = path[p].x = x[r] + t * (x[r + 1] - x[r]);
-							lines[l++].y = path[p++].y = y[r] + t * (y[r + 1] - y[r]);
+							lines[l++].y = path[p].y = y[r] + t * (y[r + 1] - y[r]);
 						} else {
 							t = (zmin - zval[r]) / (zval[0] - zval[r]);
 							lines[l].x = path[p].x = x[r] + t * (x[0] - x[r]);
-							lines[l++].y = path[p++].y = y[r] + t * (y[0] - y[r]);
+							lines[l++].y = path[p].y = y[r] + t * (y[0] - y[r]);
 						}
+						if (fabs (path[p-1].x - path[p].x) > CONTOUR_EPSILON ||
+							fabs (path[p-1].y - path[p].y) > CONTOUR_EPSILON)
+							p++;
 						if (s == 0) {
 							for (h = r + 1; h <= kmax; h++) {
-								path[p].code = ART_LINETO;
-								path[p].x = x[h];
-								path[p++].y = y[h];
+								if (fabs (path[p-1].x - x[h]) > CONTOUR_EPSILON ||
+									fabs (path[p-1].y - y[h]) > CONTOUR_EPSILON) {
+									path[p].code = ART_LINETO;
+									path[p].x = x[h];
+									path[p++].y = y[h];
+								}
 							}
 						} else {
 							for (h = r + 1; h < s; h++) {
-								path[p].code = ART_LINETO;
-								path[p].x = x[h];
-								path[p++].y = y[h];
+								if (fabs (path[p-1].x - x[h]) > CONTOUR_EPSILON ||
+									fabs (path[p-1].y - y[h]) > CONTOUR_EPSILON) {
+									path[p].code = ART_LINETO;
+									path[p].x = x[h];
+									path[p++].y = y[h];
+								}
 							}
 						}
 						s = r + 1;
-						path[p].code = ART_LINETO;
-						path[p].x = path[0].x;
-						path[p++].y = path[0].y;
+						if (fabs (path[p-1].x - path[0].x) > CONTOUR_EPSILON ||
+							fabs (path[p-1].y -path[0].y) > CONTOUR_EPSILON) {
+							path[p].code = ART_LINETO;
+							path[p].x = path[0].x;
+							path[p++].y = path[0].y;
+						} else {
+							/* use the exact values so that the polygon is closed */
+							path[p-1].x = path[0].x;
+							path[p-1].y = path[0].y;
+						}
 						path[p].code = ART_END;
-						gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+						gog_renderer_draw_polygon (rend, path, FALSE);
 						gog_renderer_pop_style (rend);
 						path[0].x = lines[l - 1].x;
 						path[0].y = lines[l - 1].y;
 						path[1].x = lines[l - 2].x;
 						path[1].y = lines[l - 2].y;
-						p = 2;
+						p = (fabs (path[0].x - path[1].x) > CONTOUR_EPSILON ||
+							fabs (path[0].y - path[1].y) > CONTOUR_EPSILON)?
+							2: 1;
 					}
+					if (fabs (path[0].x - path[1].x) < CONTOUR_EPSILON
+						&& fabs (path[0].y - path[1].y) < CONTOUR_EPSILON)
+						continue;
 					while (k < s) {
 						path[p].code = ART_LINETO;
 						path[p].x = x[k];
@@ -1362,9 +1402,10 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 					path[p].x = path[0].x;
 					path[p++].y = path[0].y;
 					path[p].code = ART_END;
+					style->outline.color = color[zmin];
 					style->fill.pattern.back = color[zmin];
 					gog_renderer_push_style (rend, style);
-					gog_renderer_draw_polygon (rend, path, TRUE, NULL);
+					gog_renderer_draw_polygon (rend, path, FALSE);
 					gog_renderer_pop_style (rend);
 				}
 			}
@@ -1372,9 +1413,9 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 	}
 	lines[l].code = ART_END;
 	gog_renderer_push_style (rend, GOG_STYLED_OBJECT (series)->style);
-	gog_renderer_draw_path  (rend, lines, NULL);
+	gog_renderer_draw_path  (rend, lines);
 	gog_renderer_pop_style (rend);
-	gog_renderer_clip_pop (rend);
+	gog_renderer_pop_clip (rend);
 	art_free (lines);
 	art_free (path);
 	g_object_unref (style);
@@ -1388,9 +1429,9 @@ gog_contour_view_class_init (GogViewClass *view_klass)
 	view_klass->render = gog_contour_view_render;
 }
 
-static GSF_CLASS (GogContourView, gog_contour_view,
-		  gog_contour_view_class_init, NULL,
-		  GOG_PLOT_VIEW_TYPE)
+GSF_DYNAMIC_CLASS (GogContourView, gog_contour_view,
+	gog_contour_view_class_init, NULL,
+	GOG_PLOT_VIEW_TYPE)
 
 /*****************************************************************************/
 
@@ -1455,15 +1496,19 @@ gog_surface_series_class_init (GogStyledObjectClass *gso_klass)
 }
 
 
-GSF_CLASS (GogSurfaceSeries, gog_surface_series,
-	   gog_surface_series_class_init, NULL,
-	   GOG_SERIES_TYPE)
+GSF_DYNAMIC_CLASS (GogSurfaceSeries, gog_surface_series,
+	gog_surface_series_class_init, NULL,
+	GOG_SERIES_TYPE)
 
 G_MODULE_EXPORT void
 go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 {
-	gog_contour_plot_get_type ();
-	xl_contour_plot_get_type ();
+	GTypeModule *module = go_plugin_get_type_module (plugin);
+	gog_contour_plot_register_type (module);
+	gog_contour_view_register_type (module);
+	gog_surface_series_register_type (module);
+	xl_contour_plot_register_type (module);
+	xl_surface_series_register_type (module);
 }
 
 G_MODULE_EXPORT void
